#include <windows.h>
#include "../current/lsapi/lsapi.h"
#include "lsutils.h"
#include "exports.h"
#include <string.h>
#include "LSClipboard.h"
#include "CClipboard.h"

#define WC_SHELLDESKTOP    "DesktopBackgroundClass"

CLSClipboard LsClipboard;
CClipboard ClipboardManager;

int g_nResolutionX, g_nResolutionY;
char* g_pszAppName = "LSClipboard";
char* g_pszVersion = "LSClipboard 2.14\0";
HWND g_hWindowList;


CLSClipboard::CLSClipboard():
m_bUpdate(true), m_bOnTop(false)
{
	strcpy(m_pszIniFile, "");
}

CLSClipboard::~CLSClipboard()
{
   DeleteObject(m_hFont);
}

void BangOnTop(HWND caller, const char* args)
{
	SetWindowPos(LsClipboard.m_hWindowMain, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	LsClipboard.m_bOnTop=TRUE;
}

void BangNotOnTop(HWND caller, const char* args)
{
	SetWindowPos(LsClipboard.m_hWindowMain, HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
	LsClipboard.m_bOnTop=FALSE;
}

void BangToggleOnTop(HWND caller, const char* args)
{
	if (LsClipboard.m_bOnTop) BangNotOnTop(caller, args);
	else BangOnTop(caller, args);
}

void BangShow(HWND caller, const char* args) 
{
	ShowWindow(LsClipboard.m_hWindowMain, SW_SHOW);
	SetFocus(LsClipboard.m_hWindowList);
	SetForegroundWindow(LsClipboard.m_hWindowList);
}

void BangHide(HWND caller, const char* args) 
{ 
	ShowWindow(LsClipboard.m_hWindowMain, SW_HIDE);
}

void BangToggle(HWND caller, const char* args)
{
	if (IsWindowVisible(LsClipboard.m_hWindowMain)) BangHide(caller, args);
	else BangShow(caller, args);
}

void BangClear(HWND caller, const char* args)
{
	SendMessage(LsClipboard.m_hWindowList, LB_RESETCONTENT, 0, 0);
	SetFocus(LsClipboard.m_hWindowList);
}

void CLSClipboard::LoadSetup(LPCSTR szPath)
{
	char temp[256] = "";

	if(GetRCString( "CBFile", LsClipboard.m_pszIniFile, "", MAX_PATH ));
	else{
		strcpy(LsClipboard.m_pszIniFile,szPath);
		strcat(LsClipboard.m_pszIniFile, "\\lsclipboard.ini");
	}

	g_nResolutionX = GetSystemMetrics(SM_CXSCREEN);
	g_nResolutionY = GetSystemMetrics(SM_CYSCREEN);

	AddBangCommand("!CBShow", BangShow);
	AddBangCommand("!CBHide", BangHide);
	AddBangCommand("!CBToggle", BangToggle);
	AddBangCommand("!CBClear", BangClear);
	AddBangCommand("!CBOnTop", BangOnTop);
	AddBangCommand("!CBNotOnTop", BangNotOnTop);
	AddBangCommand("!CBToggleOnTop", BangToggleOnTop);

	LsClipboard.m_nX = GetRCInt("CBX", 100);
	if (LsClipboard.m_nX < 0)
		LsClipboard.m_nX = g_nResolutionX + LsClipboard.m_nX;
	LsClipboard.m_nY = GetRCInt("CBY", 100);
	if (LsClipboard.m_nY < 0)
		LsClipboard.m_nY = g_nResolutionY + LsClipboard.m_nY;
	LsClipboard.m_nWidth = GetRCInt("CBWidth", 150);
	LsClipboard.m_nHeight = GetRCInt("CBHeight", 150);

	LsClipboard.m_nBorderTop = GetRCInt("CBBorderTop", 5);
	LsClipboard.m_nBorderRight = GetRCInt("CBBorderRight", 5);
	LsClipboard.m_nBorderBottom = GetRCInt("CBBorderBottom", 5);
	LsClipboard.m_nBorderLeft = GetRCInt("CBBorderLeft", 5);

	LsClipboard.m_nFontSize = GetRCInt("CBFontSize",12);
	GetRCString("CBFontName",LsClipboard.m_pszFontName,"System",256);

	GetRCString("CBBgBmp", temp, "", 256);
	if (strlen(temp))
	{
		LsClipboard.m_hbmBackGround = LoadLSImage(temp, temp);
	}

	LsClipboard.m_clrBackground = GetRCColor("CBColorListBg", 0x00FFFFFF);
	LsClipboard.m_clrFontNormal = GetRCColor("CBColorListFg", 0x00000000);
	LsClipboard.m_clrFontSelected = GetRCColor("CBColorListFgSel", 0x00FFFFFF);

}

void CLSClipboard::ApplyFont(DWORD dwCharSet)
{
   m_hFont = CreateFont(m_nFontSize, 0, 0, 0, 200, false, false, false, dwCharSet, 0, 0, PROOF_QUALITY, 0, m_pszFontName);
   SendMessage(m_hWindowList, WM_SETFONT, (WPARAM)m_hFont, TRUE);
}

int initModuleEx(HWND parent, HINSTANCE dllInst, LPCSTR szPath)
{
	WNDCLASS wc;
	int msgs[] = {LM_GETREVID, 0};

	LsClipboard.m_hWindowParent = parent;

	memset(&wc,0,sizeof(wc));
	wc.lpfnWndProc = CLSClipboard::WndProc;
	wc.hInstance = dllInst;
	wc.lpszClassName = g_pszAppName;
	
	if (!RegisterClass(&wc)) 
	{
		MessageBox(parent, "Error: Could not register window class", g_pszAppName, MB_OK | MB_ICONERROR);
		return 1;
	}
 
	LsClipboard.LoadSetup(szPath);

	if (GetRCBool("CBAlwaysOnTop", TRUE))
	{
		LsClipboard.m_bOnTop=TRUE;
	}

	 HWND parentWindow = NULL;
	 parentWindow = GetParentSettings(LsClipboard.m_bOnTop);

	 LsClipboard.m_hWindowMain = CreateWindowEx(LsClipboard.m_bOnTop ? WS_EX_TOOLWINDOW | WS_EX_TOPMOST : WS_EX_TOOLWINDOW, 
							  g_pszAppName, g_pszAppName, 
							  WS_POPUP | WS_CLIPCHILDREN | WS_CLIPSIBLINGS , 
							  LsClipboard.m_nX, LsClipboard.m_nY,
							  LsClipboard.m_nWidth, LsClipboard.m_nHeight, 
							  parentWindow, NULL, dllInst, NULL);

	if (!LsClipboard.m_hWindowMain) return 1;

	SetWindowLong(LsClipboard.m_hWindowMain, GWL_USERDATA, 0x49474541);

	DWORD dwStyle;
	dwStyle = WS_CHILD | WS_VISIBLE  | LBS_HASSTRINGS | LBS_NOTIFY | LBS_OWNERDRAWVARIABLE;
	//new 20030102
	if (!GetRCBool("CBNoVerticalScroll", TRUE))
	{
		dwStyle = dwStyle | WS_VSCROLL;
	}
	if (GetRCBool("CBSortList", TRUE))
	{
		dwStyle = dwStyle | LBS_SORT;
	}

	if (GetRCBool("CBDisableKeys", TRUE))
	{
		LsClipboard.m_bNoKeyboardInput = true;
	}
	else
	{
		LsClipboard.m_bNoKeyboardInput = false;
		dwStyle = dwStyle | LBS_WANTKEYBOARDINPUT;
	}
	//end new 20030102

	g_hWindowList = LsClipboard.m_hWindowList = CreateWindow("LISTBOX", "", dwStyle,
							LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop,
							(LsClipboard.m_nWidth)-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft, (LsClipboard.m_nHeight)-LsClipboard.m_nBorderBottom-LsClipboard.m_nBorderTop,
							LsClipboard.m_hWindowMain, NULL, dllInst, NULL); 
	LsClipboard.LoadFromFile(LsClipboard.m_pszIniFile);

	ClipboardManager.AttachToClipboard(LsClipboard.m_hWindowMain);

	SendMessage(LsClipboard.m_hWindowParent, LM_REGISTERMESSAGE, (WPARAM)LsClipboard.m_hWindowMain, (LPARAM)msgs);
	LsClipboard.ApplyFont(DEFAULT_CHARSET);

	if (GetRCBool("CBStartHidden", FALSE))
	{
		ShowWindow(LsClipboard.m_hWindowMain, SW_SHOWNORMAL);
	}
	
	return 0;
}

int quitModule(HINSTANCE dll)
{
	int msgs[] = {LM_GETREVID, 0};

	LsClipboard.SaveToFile(LsClipboard.m_pszIniFile);

	RemoveBangCommand("!CBShow");
	RemoveBangCommand("!CBHide");
	RemoveBangCommand("!CBToggle");
	RemoveBangCommand("!CBClear");
	RemoveBangCommand("!CBOnTop");
	RemoveBangCommand("!CBNotOnTop");
	RemoveBangCommand("!CBToggleOnTop");

	ClipboardManager.DetachFromClipboard();
	
	SendMessage(LsClipboard.m_hWindowParent, LM_UNREGISTERMESSAGE, (WPARAM)LsClipboard.m_hWindowMain, (LPARAM)msgs);
	DestroyWindow(LsClipboard.m_hWindowMain);

	UnregisterClass(g_pszAppName, dll);

	return 0;
}

LRESULT CLSClipboard::OnGetRevId(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	LPSTR buf = (LPSTR)(lParam);

	switch (wParam)
	{
		case 0:
			sprintf(buf, "%s", g_pszVersion);
		break;
		case 1:
			sprintf(buf, "%s", g_pszVersion);
		break;
		default:
			sprintf(buf, "%s", g_pszVersion);
	}
	return strlen(buf);
}

LRESULT CLSClipboard::OnEraseBackground(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT CLSClipboard::OnNCHitTest(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	int x = LOWORD(lParam);
	int y = HIWORD(lParam);
	RECT r;
	long htx=0;
	long hty=0;
	long ht=0;

	GetWindowRect(hWindow, &r);

	if ((y - r.top) < LsClipboard.m_nBorderTop) hty=HTCAPTION;
	else if ((y-r.top) > r.bottom-r.top-LsClipboard.m_nBorderBottom) hty=HTBOTTOM;
	else hty=0;
			
	if ((x-r.left) < LsClipboard.m_nBorderLeft) htx=HTLEFT;
	else if ((x-r.left) > r.right-r.left-LsClipboard.m_nBorderRight) htx=HTRIGHT;
	else htx=0;

	if (htx && hty)
	{
		if (hty == HTBOTTOM)
		{
			if (htx == HTLEFT) ht=HTBOTTOMLEFT;
			else if (htx == HTRIGHT) ht = HTBOTTOMRIGHT;
		}
	}
	else if (htx) ht = htx;
	else if (hty) ht = hty;
	else ht=HTCLIENT;

	return ht;
}

LRESULT CLSClipboard::OnPaint(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	RECT r;
	static RECT or;
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(hWindow, &ps);
	HDC buf = CreateCompatibleDC(NULL);
	HDC src = CreateCompatibleDC(NULL);
	HBITMAP bufbmp;
	HBITMAP oldsrc, oldbuf;

	GetClientRect(hWindow, &r);
	bufbmp = CreateCompatibleBitmap(hdc, r.right, r.bottom);
	oldbuf = (HBITMAP)SelectObject(buf, bufbmp);

	if (LsClipboard.m_hbmBackGround)
	{
		BITMAP bmp;
					
		GetObject(LsClipboard.m_hbmBackGround, sizeof(bmp), &bmp);
		oldsrc = (HBITMAP)SelectObject(src, LsClipboard.m_hbmBackGround);

		// Top Left Corner
		StretchBlt(buf, 0, 0, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, src, 0, 0, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, SRCCOPY);

		// Top Right Corner
		StretchBlt(buf, r.right-LsClipboard.m_nBorderRight, 0, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, src, bmp.bmWidth-LsClipboard.m_nBorderRight, 0, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, SRCCOPY);

		// Top
		StretchBlt(buf, LsClipboard.m_nBorderLeft, 0, r.right-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft, LsClipboard.m_nBorderTop, src, LsClipboard.m_nBorderLeft, 0, bmp.bmWidth-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft, LsClipboard.m_nBorderTop, SRCCOPY);

		// Bottom Left Corner
		StretchBlt(buf, 0, r.bottom-LsClipboard.m_nBorderBottom, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, src, 0, bmp.bmHeight-LsClipboard.m_nBorderBottom, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, SRCCOPY);

		// Bottom Right Corner
		StretchBlt(buf, r.right-LsClipboard.m_nBorderRight, r.bottom-LsClipboard.m_nBorderBottom, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, src, bmp.bmWidth-LsClipboard.m_nBorderRight, bmp.bmHeight-LsClipboard.m_nBorderBottom, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, SRCCOPY);

		// Bottom
		StretchBlt(buf, LsClipboard.m_nBorderLeft, r.bottom-LsClipboard.m_nBorderBottom, r.right-LsClipboard.m_nBorderLeft-LsClipboard.m_nBorderRight, LsClipboard.m_nBorderBottom, src, LsClipboard.m_nBorderLeft, bmp.bmHeight-LsClipboard.m_nBorderBottom, bmp.bmWidth-LsClipboard.m_nBorderLeft-LsClipboard.m_nBorderRight, LsClipboard.m_nBorderBottom, SRCCOPY);
						
		// Left
		StretchBlt(buf, 0, LsClipboard.m_nBorderTop, LsClipboard.m_nBorderLeft, r.bottom-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, src, 0, LsClipboard.m_nBorderTop, LsClipboard.m_nBorderLeft, bmp.bmHeight-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, SRCCOPY);
						
		// Right
		StretchBlt(buf, r.right-LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, LsClipboard.m_nBorderRight, r.bottom-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, src, bmp.bmWidth-LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, LsClipboard.m_nBorderRight, bmp.bmHeight-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, SRCCOPY);
						
		// Client
		StretchBlt(buf, LsClipboard.m_nBorderLeft, LsClipboard.m_nBorderTop, r.right-LsClipboard.m_nBorderLeft-LsClipboard.m_nBorderRight, r.bottom-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, src, LsClipboard.m_nBorderLeft, LsClipboard.m_nBorderTop, bmp.bmWidth-LsClipboard.m_nBorderLeft-LsClipboard.m_nBorderRight, bmp.bmHeight-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, SRCCOPY);

		if (r.right != or.right || r.bottom != or.bottom)
		{
			SetRect(&or, r.left, r.top, r.right, r.bottom);
			LsClipboard.SetWindowBitmapRgn(hWindow, buf);
		}
	}

	BitBlt(hdc, 0, 0, r.right, r.bottom, buf, 0, 0, SRCCOPY);

	SelectObject(buf, oldbuf);
	DeleteDC(buf);
	DeleteObject(bufbmp);
	DeleteObject(oldbuf);
	SelectObject(src, oldsrc);
	DeleteDC(src);
	DeleteObject(oldsrc);
	EndPaint(hWindow, &ps);
	DeleteDC(hdc);
	
	return 0;
}

LRESULT CLSClipboard::OnMeasureItem(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	LPMEASUREITEMSTRUCT mis = (LPMEASUREITEMSTRUCT)lParam;
	RECT r;
	int len = SendMessage(LsClipboard.m_hWindowList, LB_GETTEXTLEN, mis->itemID, 0);
	char* temp = new char[len];
	SendMessage(LsClipboard.m_hWindowList, LB_GETTEXT, (WPARAM)mis->itemID, (LPARAM)temp);
	SetRect(&r, 0, 0, (LsClipboard.m_nWidth)-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft, 0);
	DrawText(GetDC(LsClipboard.m_hWindowList), temp, -1, &r, DT_CALCRECT | DT_EDITCONTROL | DT_WORDBREAK | DT_EXPANDTABS);

	mis->itemWidth = (LsClipboard.m_nWidth)-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft;
	mis->itemHeight = r.bottom;
	return TRUE;
}

LRESULT CLSClipboard::OnDrawItem(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	LPDRAWITEMSTRUCT dis = (LPDRAWITEMSTRUCT)lParam;
					
	HBRUSH hbr = CreateSolidBrush(LsClipboard.m_clrBackground);
	RECT r;
	GetClientRect(LsClipboard.m_hWindowList, &r);
	FillRect(dis->hDC, &r, hbr);
	DeleteObject(hbr);

	int count = SendMessage(LsClipboard.m_hWindowList, LB_GETCOUNT, 0, 0);
	int cursel = SendMessage(LsClipboard.m_hWindowList, LB_GETCURSEL, 0, 0);

	for (int x=0; x<count; x++)
	{
		RECT ir;
		SendMessage(LsClipboard.m_hWindowList, LB_GETITEMRECT, x, (LPARAM)&ir);

		int len = SendMessage(LsClipboard.m_hWindowList, LB_GETTEXTLEN, x, 0);
		char* temp = new char[len];
		SendMessage(LsClipboard.m_hWindowList, LB_GETTEXT, (WPARAM)x, (LPARAM)temp);

		SetBkMode(dis->hDC, TRANSPARENT);

		if (x == cursel)
		{
			SetTextColor(dis->hDC, LsClipboard.m_clrFontSelected);
		}
		else
		{
			SetTextColor(dis->hDC, LsClipboard.m_clrFontNormal);
		}
		
		DrawText(dis->hDC, temp, -1, &ir, DT_LEFT | DT_EDITCONTROL | DT_WORDBREAK | DT_EXPANDTABS);
	}
	return TRUE;
}

void CLSClipboard::PutSelectedInClipboard()
{
	int nCurrentSelecteed = LsClipboard.GetCurrentIndex();
	if (nCurrentSelecteed > -1)
	{
		int nLength = SendMessage(m_hWindowList, LB_GETTEXTLEN, (WPARAM)nCurrentSelecteed, 0);
		char* pszTemp = new char[nLength];
		SendMessage(m_hWindowList, LB_GETTEXT, (WPARAM)nCurrentSelecteed, (LPARAM)pszTemp);
		ClipboardManager.InsertInClipboard(pszTemp);
		delete[] pszTemp;
	}
}


LRESULT CLSClipboard::OnKeyToItem(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	if(LsClipboard.m_bNoKeyboardInput)
	{
		return -2;
	}
	
	if (LOWORD(wParam) == VK_DELETE)
	{
		int cursel = LsClipboard.GetCurrentIndex();
		SendMessage(g_hWindowList, LB_DELETESTRING, (WPARAM)cursel, 0);
		SendMessage(g_hWindowList, LB_SETCURSEL, cursel, 0);
		return -2;
	}
	else if (LOWORD(wParam) == VK_RETURN)
	{
		LsClipboard.PutSelectedInClipboard();
		return -2;
	}
	return -1;
}

LRESULT CLSClipboard::OnChangedCBChain(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	ClipboardManager.OnChangedCBChain(WM_CHANGECBCHAIN, wParam, lParam);
	return 0;
}

LRESULT CLSClipboard::OnDrawClipboard(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	if (LsClipboard.m_bUpdate)
	{
		if (OpenClipboard(LsClipboard.m_hWindowMain))
		{
			if (IsClipboardFormatAvailable(CF_TEXT))
			{
				HANDLE cbdata = GetClipboardData(CF_TEXT);

				if (cbdata)
				{
					LPVOID pData;
					pData = GlobalLock(cbdata);

					bool bFound = false;
					char *pszBuffer;
					int nPosition = 0;
					int nCount = SendMessage(LsClipboard.m_hWindowList, LB_GETCOUNT, 0, 0);
					if(nCount > 0)
					{
						while ( !bFound && (nPosition < nCount))
						{
							pszBuffer = (char*) malloc (SendMessage( LsClipboard.m_hWindowList, LB_GETTEXTLEN, nPosition, 0)*sizeof(char));
							SendMessage( LsClipboard.m_hWindowList, LB_GETTEXT, nPosition, (LPARAM) pszBuffer);
							if(!strcmp((char*)pData, pszBuffer))
							{
								bFound = true;
							}
							else
							{
								++nPosition;
							}
							free(pszBuffer);
						}
					}
					if(!bFound)
					{
						nPosition = SendMessage(LsClipboard.m_hWindowList, LB_ADDSTRING, 0, (LPARAM)pData);
					}
					SendMessage(LsClipboard.m_hWindowList, LB_SETTOPINDEX, (WPARAM)nPosition, 0);
					SendMessage(LsClipboard.m_hWindowList, LB_SETCURSEL, nPosition, 0);

					GlobalUnlock(cbdata);
				}
			}
			CloseClipboard();			
		}
	}
	else
		LsClipboard.m_bUpdate=TRUE;
	return SendMessage(LsClipboard.m_hWindowNextInList, WM_DRAWCLIPBOARD, wParam, lParam);
}

LRESULT CLSClipboard::OnCommand(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	if (LBN_DBLCLK == HIWORD(wParam))
	{
		int cursel = SendMessage(LsClipboard.m_hWindowList, LB_GETCURSEL, 0, 0);
		if (cursel > -1)
		{
			int len = SendMessage(LsClipboard.m_hWindowList, LB_GETTEXTLEN, (WPARAM)cursel, 0);
			char* temp = new char[len];
			SendMessage(LsClipboard.m_hWindowList, LB_GETTEXT, (WPARAM)cursel, (LPARAM)temp);
			if (OpenClipboard(LsClipboard.m_hWindowMain))
			{
				EmptyClipboard();
				HGLOBAL hglbCopy = GlobalAlloc(GMEM_MOVEABLE, (len + 1) * sizeof(TCHAR)); 
				if (hglbCopy)
				{
					LPTSTR lptstrCopy;
					lptstrCopy = (LPTSTR)GlobalLock(hglbCopy); 
					memcpy(lptstrCopy, temp, 
					len * sizeof(TCHAR)); 
					lptstrCopy[len] = (TCHAR)0;
					GlobalUnlock(hglbCopy);
					LsClipboard.m_bUpdate=FALSE;
					SetClipboardData(CF_TEXT, hglbCopy);
				}
				CloseClipboard();
			}
		}
	}
	return 0;
}

LRESULT CLSClipboard::OnWindowPosChanging(HWND hWindow, WPARAM wParam, LPARAM lParam)
{
	WINDOWPOS* pos = (WINDOWPOS*)lParam;
					
	if (!(pos->flags & SWP_NOMOVE))
	{		
		if (pos->x + LsClipboard.m_nWidth >= g_nResolutionX - 5)
			pos->x = g_nResolutionX - LsClipboard.m_nWidth;
		else if (pos->x <= 5)
			pos->x = 0;
		if (pos->y + LsClipboard.m_nHeight >= g_nResolutionY - 5)
			pos->y = g_nResolutionY - LsClipboard.m_nHeight;
		else if (pos->y <= 5) pos->y = 0;

		LsClipboard.m_nX = pos->x;
		LsClipboard.m_nY = pos->y;
	}

	if ((LsClipboard.m_nWidth != pos->cx || LsClipboard.m_nHeight != pos->cy) && (pos->cx && pos->cy))
	{
		LsClipboard.m_nWidth = pos->cx;
		LsClipboard.m_nHeight = pos->cy; 

		SetWindowPos(LsClipboard.m_hWindowList, NULL, LsClipboard.m_nBorderRight, LsClipboard.m_nBorderTop, (LsClipboard.m_nWidth)-LsClipboard.m_nBorderRight-LsClipboard.m_nBorderLeft, (LsClipboard.m_nHeight)-LsClipboard.m_nBorderTop-LsClipboard.m_nBorderBottom, SWP_NOZORDER);
		InvalidateRect(hWindow, NULL, TRUE);
	}
	
	return 0;
}

LRESULT CALLBACK CLSClipboard::WndProc(HWND hWindow, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
		case LM_GETREVID:
		{
			return OnGetRevId(hWindow, wParam, lParam);
		}
		case WM_ERASEBKGND: 
		{
			return OnEraseBackground(hWindow, wParam, lParam);
		}
		case WM_NCHITTEST: 
		{
			return OnNCHitTest(hWindow, wParam, lParam);
		}

		case WM_NCPAINT:
		case WM_PAINT:
		{
			return OnPaint(hWindow, wParam, lParam);
		}

		case WM_MEASUREITEM:
		{
			return OnMeasureItem(hWindow, wParam, lParam);
		}

		case WM_DRAWITEM:
		{
			return OnDrawItem(hWindow, wParam, lParam);
		}

		case WM_VKEYTOITEM:
		{
			return OnKeyToItem(hWindow, wParam, lParam);
		}

		case WM_CHANGECBCHAIN:
		{
			return OnChangedCBChain(hWindow, wParam, lParam);
		}

		case WM_DRAWCLIPBOARD:
		{
			return OnDrawClipboard(hWindow, wParam, lParam);
		}

		case WM_COMMAND:
		{
			return OnCommand(hWindow, wParam, lParam);
		}

		case WM_WINDOWPOSCHANGING:
		{
			return OnWindowPosChanging(hWindow, wParam, lParam);
		}
	}

	return DefWindowProc(hWindow,msg, wParam,lParam);
}

void CLSClipboard::SetWindowBitmapRgn(HWND hwnd, HDC hdc)
{
	if (hwnd && hdc)
	{
		int x=0, y=0;
		HRGN hTransRgn=NULL;
		HRGN hMainRgn=NULL;
		RECT r;
		GetClientRect(hwnd, &r);
		hMainRgn = CreateRectRgn(0, 0, r.right, r.bottom);

		// Top Border
		for (y=0; y <= LsClipboard.m_nBorderTop; y++)
		{
			for (x=0; x < r.right; x++)
			{
				COLORREF c = GetPixel(hdc, x, y);
				if (c == RGB(255, 0, 255))
				{
					HRGN hTempRgn = CreateRectRgn(x, y, x+1, y+1);
					if (!hTransRgn) hTransRgn = CreateRectRgn(x, y, x+1, y+1);
					CombineRgn(hTransRgn, hTransRgn, hTempRgn, RGN_OR);
					DeleteObject(hTempRgn);
				}
			}
		}

		// Left and Right Borders
		for (y=LsClipboard.m_nBorderTop; y <= r.bottom-LsClipboard.m_nBorderBottom; y++)
		{
			for (x=0; x < LsClipboard.m_nBorderLeft; x++)
			{
				COLORREF c = GetPixel(hdc, x, y);
				if (c == RGB(255, 0, 255))
				{
					HRGN hTempRgn = CreateRectRgn(x, y, x+1, y+1);
					if (!hTransRgn) hTransRgn = CreateRectRgn(x, y, x+1, y+1);
					CombineRgn(hTransRgn, hTransRgn, hTempRgn, RGN_OR);
					DeleteObject(hTempRgn);
				}
			}

			for (x=r.right-LsClipboard.m_nBorderRight; x < r.right; x++)
			{
				COLORREF c = GetPixel(hdc, x, y);
				if (c == RGB(255, 0, 255))
				{
					HRGN hTempRgn = CreateRectRgn(x, y, x+1, y+1);
					if (!hTransRgn) hTransRgn = CreateRectRgn(x, y, x+1, y+1);
					CombineRgn(hTransRgn, hTransRgn, hTempRgn, RGN_OR);
					DeleteObject(hTempRgn);
				}
			}
		}
		
		// Bottom Border
		for (y=r.bottom-LsClipboard.m_nBorderBottom; y < r.bottom; y++)
		{
			for (x=0; x < r.right; x++)
			{
				COLORREF c = GetPixel(hdc, x, y);
				if (c == RGB(255, 0, 255))
				{
					HRGN hTempRgn = CreateRectRgn(x, y, x+1, y+1);
					if (!hTransRgn) hTransRgn = CreateRectRgn(x, y, x+1, y+1);
					CombineRgn(hTransRgn, hTransRgn, hTempRgn, RGN_OR);
					DeleteObject(hTempRgn);
				}
			}
		}
		
		CombineRgn(hMainRgn, hMainRgn, hTransRgn, RGN_DIFF);
		SetWindowRgn(hwnd, hMainRgn, TRUE);

		DeleteObject(hTransRgn);
	}
	else
	{
		RECT r;
		HRGN rgn;
		GetClientRect(hwnd, &r);
		rgn = CreateRectRgn(r.left, r.top, r.right, r.bottom);
		SetWindowRgn(hwnd, rgn, FALSE);
	}
}

void CLSClipboard::SaveToFile(const char* ini)
{
	FILE* file = fopen(ini, "w");
	char temp[256] = "";
	int x, count;
	
	if (file)
	{
		count = SendMessage(LsClipboard.m_hWindowList, LB_GETCOUNT, 0, 0);
		//20030102
		//for (x=0; x<count-1; x++)
		for (x=0; x<count; x++) //would prevent from saving the latest entry
		{
			int len = SendMessage(LsClipboard.m_hWindowList, LB_GETTEXTLEN, x, 0);
			char* text = new char[len];
			SendMessage(LsClipboard.m_hWindowList, LB_GETTEXT, (WPARAM)x, (LPARAM)text);

			fprintf(file, "****====****====****====\n");
			fprintf(file, "%s\n", text);
			fprintf(file, "****====****====****====\n");
		}
		fclose(file);
	}
}

void CLSClipboard::LoadFromFile(const char* ini)
{
	FILE* file = fopen(ini, "r");
	char temp[256] = "";

	if (file)
	{
		while (!feof(file))
		{
			fgets(temp, 256, file);
			
			if (!strcmp(temp, "****====****====****====\n"))
			{
				char text[2048] = "";
				fgets(temp, 256, file);
				while (strcmp(temp, "****====****====****====\n") && !feof(file))
				{
					if (strlen(text) + strlen(temp) < 2048)
					{
						strcat(text, temp);
					}
					fgets(temp, 256, file);
				}
				
				if (strlen(text))
				{
					if (text[strlen(text)-1] == '\n')
						text[strlen(text)-1] = '\0';
					int index = SendMessage(LsClipboard.m_hWindowList, LB_ADDSTRING, 0, (LPARAM)text);
					SendMessage(LsClipboard.m_hWindowList, LB_SETTOPINDEX, (WPARAM)index, 0);
					SendMessage(LsClipboard.m_hWindowList, LB_SETCURSEL, index, 0);
				}
			}
		}
		fclose(file);
	}
}

int CLSClipboard::GetCurrentIndex()
{
	return SendMessage(m_hWindowList, LB_GETCURSEL, 0, 0);
}

int CLSClipboard::SetCurrentIndex(int nIndex)
{
	SendMessage(LsClipboard.m_hWindowList, LB_SETTOPINDEX, (WPARAM)nIndex, 0);
	SendMessage(LsClipboard.m_hWindowList, LB_SETCURSEL, nIndex, 0);
	return nIndex;
}

int CLSClipboard::GetCurrentCount()
{
	return SendMessage(LsClipboard.m_hWindowList, LB_GETCOUNT, 0, 0);
}
